/*
 * CCameraKeyPathPoint.h
 * $Id: 
 *
 * Copyright (C) 2001 Alexander Buck
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * As a special exception to the GPL, the QGLViewer authors (Markus
 * Janich, Michael Meissner, Richard Guenther, Alexander Buck and Thomas
 * Woerner) give permission to link this program with Qt (non-)commercial
 * edition, and distribute the resulting executable, without including
 * the source code for the Qt (non-)commercial edition in the source
 * distribution.
 *
 */

//  Description : Class CCameraKeyPathPoint.h
//  Purpose     : Provides a camerapathpoint

#ifndef CCAMERAKEYPATHPOINT_H
#define CCAMERAKEYPATHPOINT_H


// Qt
///////
#include <qstring.h>

// QGLViewer
//////////////
#include <CCamera.h>

// Own
///////////
#include "CCameraKeyPathAttributes.h"


/**Provides a camerakeypathpoint
  *
  * @author Alexander Buck
  */


class CCameraKeyPathPoint
{
 public:
  
  //////////////////
  // CONSTRUCTORS //
  //////////////////

  /** default constructor.
    * Creates a default camera, default attributes, and 
    * sets the name to "new camera".
    */
  CCameraKeyPathPoint() :
    m_camera(CCamera()),
    m_attributes(CCameraKeyPathAttributes()),
    m_qName("new camera")
    {};

  /** Constructor using given camera, attributes and name. */ 
  CCameraKeyPathPoint(CCamera camera, CCameraKeyPathAttributes atts, QString qName )
    : m_camera(camera),
      m_attributes(atts),
      m_qName(qName)
    {};

  /** default destructor. */
  ~CCameraKeyPathPoint() {};
  
  /////////////////////
  // PUBLIC METHODS ///
  /////////////////////

  /** Set camera to given camera. */
  void setCamera(CCamera);
  
  /** Set attributes to given attributes. */
  void setAttributes(CCameraKeyPathAttributes);

  /** Set attributes to given attributes. */
  void setAttributes(int nFrames, float rfTension, float rfCon, float rfBias);
  
  /** Set name to given name. */
  void setName(QString);

  /** Returns a pointer to a camera.
    * To change camera and attribute members.
    */
  CCamera* changeCamera()
    {
      return &m_camera;
    }

  /** Returns camera. */
  CCamera getCamera() const 
    {
      return m_camera;
    };

  /** Returns attributes. */
  CCameraKeyPathAttributes getAttributes() const
    {
      return m_attributes;
    };

  /** Returns name. */
  QString getName() const
    {
      return m_qName;
    };

protected:

  ///////////////////////
  // PROTECTED MEMBERS //
  ///////////////////////

  /** Current camera. */
  CCamera m_camera;    

  /** Current attributes. */
  CCameraKeyPathAttributes m_attributes;  

  /** Current name. */
  QString m_qName;   
};

#endif // CCAMERAKEYPATHPOINT_H
